<?php

use PracticalAfas\Connection;

/**
 * Fetches data through an AFAS GetConnector.
 *
 * Other modules can create jobs and make them available to cron / admin UI, by
 * - installing the 'drunkins' Drupal contrib module;
 * - using this class as a fetcher inside their job class (by e.g. subclassing
 *   DrunkinsJob), or subclassing this class,
 * - exposing their job through hook_queue_info().
 * See https://www.drupal.org/project/drunkins for more context.
 *
 * WATCH OUT when turning on opt_fetcher_timestamp: this is fundamentally
 * incompatible with this fetcher because of the required 'fetcher_limit' (or
 * in AFAS parlance: 'take') parameter that must be added. Because of this, you
 * theoretically don't know for sure whether the query result has been
 * truncated. If you (as a programmer) are sure that in practice this is not the
 * case, you must set $context['afas_fetcher_limit_does_not_limit_result']
 * before calling parent::start() / fetchItems(), otherwise feedIsComplete()
 * returns FALSE.
 */
class AfasGetConnector extends DrunkinsFetcher {

  /* Settings in $this->settings, used by this class:
   * - job_id:              defined in parent
   *
   * - afas_api_client_class (string):
   *   The name of the client class to use. Optional; by default the one from
   *   Drupal's global AFAS API admin screen will be taken.
   * - afas_client_options (array):
   *   Configuration options for the client. Optional ONLY IF the class'
   *   constructor knows where to find its own configuration. (Which is the case
   *   for standard Drupal-admin configured client classes.)
   *
   * - afas_connector (string):
   *   The name of the GetConnector to use for fetching data.
   * - afas_skip (integer):
   *   The 'skip' parameter to a GetConnector. (The accompanying required 'take'
   *   parameter is taken from the 'fetcher_limit' setting.)
   *
   * - afas_filter (array):
   *   Filter to use. See comments (all types are supported) at
   *   \PracticalAfas\Connection::parseFilters().
   * - afas_adv_filter (string):
   *   Filter to use (one 'array level' only), as PHP code. This is only useful
   *   for the per-process settingsForm; use 'afas_filter' instead.
   * - opt_afas_adv_filter: (boolean):
   *   Enables 'afas_adv_filter' setting in settingsForm.
   *
   * - afas_updated_field (string):
   *   AFAS field containing 'updated' date values.
   *   Used only when 'fetcher_timestamp_from' is set in context (by the parent
   *   class), which signifies that a date filter should be added. This class'
   *   inputFilter() method does that automatically, to prevent any child class
   *   from forgetting this. But it needs to know the name of the 'updated'
   *   field for this.
   *   If not filled, a message will be logged.
   *
   * - afas_include_empty_fields (boolean):
   *   include empty fields in AFAS output.
   *
   * - fetcher_timeout (int):
   *   response_timeout to set for AfasNusoapClient. (Has no effect on
   *   AfasSoapClient. It doesn't matter, as long as we keep using CURL for it.)
   * - fetcher_tries (int): <==== NOT FUNCTIONING AT THE MOMENT.
   *   number of times to try connecting at timeout.
   *
   */

  // Optional extra filter. (Added for convenience; it's perfectly possible for
  // classes to do their own filtering - but this way they can 'hardcode' an
  // extra filter for their connector in their __construct function.)
  // Needs to be a single-level array (i.e. can contain multiple fields but only
  // one operator).
  // @deprecated - use settings[afas_filter].
  protected $extra_filter;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $settings = array()) {
    parent::__construct($settings);

    if (!isset($this->settings['opt_afas_adv_filter'])) {
      $this->settings['opt_afas_adv_filter'] = user_access('access drunkins advanced');
    }
  }

  /// Interface functions

  /**
   * {@inheritdoc}
   */
  public function settingsForm() {
    $form = parent::settingsForm();

    if (!empty($this->settings['opt_afas_adv_filter'])) {

      // There's a slight chance that the fieldset has not been defined yet
      // (creating #weight issues), see DrunkinsFetcher.
      if (!isset($form['selection'])) {
        $form['selection'] = array(
          '#type'   => 'fieldset',
          '#title'  => t('Data selection'),
          '#weight' => 10,
        );
      }

      // Potential security hole; this edit screen must only be available for admins.
      $form['selection']['afas_adv_filter'] = array(
        '#type'          => 'textarea',
        '#rows'          => 5,
        '#title'         => t('Advanced filter'),
        '#description'   => "Fill a comma separated set of array definitions if you want to apply other filters.<br/>
One example, to get only products that changed in the past week:<br/><em>array(<br/>
'SOME-CHANGED-DATE' => '" . date('Y-m-d\TH:i:s', time() - 7 * 86400) . "',<br/>
'#op' => \\PracticalAfas\\Connection::OP_LARGER_OR_EQUAL,<br/>
),</em>",
        '#default_value' => "array(
),",
        '#weight'        => 12,
        '#states' => array('enabled' => array(':input[name="cache_items"]' => array('!value' => '2'))),
      );

      // Extend the description of the 'cache' options (from DrunkinsFetcher),
      // IF there are cached items.
      if (isset($form['selection']['cache_items']['#options'][2]) && !empty($form['selection']['fetcher_single_item'])) {
        $form['selection']['cache_items']['#description'] .= ' ' . t('One cached item can be processed by with the \'!single_item\' option.', array('!single_item' => $form['selection']['fetcher_single_item']['#title']));
      }
    }

    // Checking the textfield is equivalent to checking the 'opt_limit' setting.
    // We are counting on the default behavior of this setting being turned on.
    if (empty($form['selection']['fetcher_limit'])) {
      drupal_set_message("The 'limit' selector is gone from this screen; this will likely mean that this job will throw an exception when starting!", 'warning');
    }
    else {
      $form['selection']['fetcher_limit']['#description'] .= ' ' . t('PLEASE NOTE: a non-zero value is mandated by AFAS.');
      // If this is explicitly set to zero, in the end the SoapAppClient will
      // throw an exception during our start() call.
      $form['selection']['fetcher_limit']['#default_value']  = !empty($this->settings['fetcher_limit']) ? $this->settings['fetcher_limit'] : 1000;
      // @todo instead of implement a mandatory limit, implement a 'paging'
      //   system with a mandatory limit-per-request which loops. I have an
      //   example in an Exact fetcher.
      $form['selection']['afas_skip'] = array(
        '#type'        => 'textfield',
        '#title'       => t('Skip items'),
        '#description' => t('Skip a number of items in the list, before taking the number of items specified below here.'),
        '#weight'      => 5,
      );
      if (!isset($this->settings['pkey'])) {
        // We want the user to use skip/take instead of effectively using the
        // same functionality on our side. (But not if the 'single_item' refers
        // to an actual ID value.)
        unset($form['selection']['fetcher_single_item']);
      }
    }

//    $form['fetcher_tries'] = array(
//      '#type'          => 'textfield',
//      '#title'         => t('Tries'),
//      '#description'   => 'The number of times to repeatedly try fetching AFAS data, in case of timeouts',
//      '#default_value' => 1,
//      '#size'          => 3,
//      '#weight'        => 97,
//    );

    return $form;
  }

  /// Extra methods called by parent class / to call by subclasses.

  /**
   * {@inheritdoc}
   */
  public function checkCache() {
    $items = parent::checkCache();
    if ($items) {
      // Implement our 'skip' function. Notes (which should be obvious):
      // - this works because checkCache() always returns all items at once and
      //   does not loop;
      // - if you've fetched items using a 'skip' setting, that does not mean
      //   the setting should be applied again. To get the same items from the
      //   cache later, set skip=0.
      if (!empty($this->settings['afas_skip'])) {
        if (!is_numeric($this->settings['afas_skip'])) {
          throw new RuntimeException(t("Invalid 'afas_skip' setting passed to inputFilter(): %p", array('%p' => drupal_json_encode($this->settings['afas_skip']))));
        }
        $items = array_slice($items, $this->settings['afas_skip']);
      }
    }

    return $items;
  }

  /**
   * {@inheritdoc}
   */
  public function feedIsComplete(array &$context, $ignore_timestamp = FALSE) {
    // Copy the code from the parent here, but add inputFilter() + afas_skip and
    // incorporate extra context check into the 'fetcher_limit' to allow the
    // caller to specify that the result is complete - since 'fetcher_limit' is
    // never 0/false in practice.
    return (empty($this->settings['fetcher_limit']) || !empty($context['afas_fetcher_limit_does_not_limit_result']))
           && empty($this->settings['fetcher_single_item'])
           && !$this->usesCachedItems()
           && ($ignore_timestamp ||
               // Check if a timestamp would be set OR if a timestamp is already
               // set (which a subclass could be doing by itself).
               ($this->getStartTimestamp() == 0 && empty($context['fetcher_timestamp_from'])))
           && empty($this->settings['afas_skip'])
           && !$this->inputFilter($context);
  }

  /**
   * {@inheritdoc}
   *
   * Modifiers:
   * - afas_return_xml in context (nonempty value returns SimpleXML, not array)
   * - afas_include_empty_fields setting
   */
  protected function fetchItems(array &$context) {
    $client_class = !empty($this->settings['afas_client_class']) ? $this->settings['afas_client_class']
      : variable_get('afas_api_client_class', 'DrupalSoapAppClient');
    $client_options = isset($this->settings['afas_client_options']) && is_array($this->settings['afas_client_options']) ? $this->settings['afas_client_options'] : array();
    if (!empty($this->settings['fetcher_timeout']) && is_numeric($this->settings['fetcher_timeout'])) {
      // Option only works for AfasNusoapClient but that's fine.
      $client_options['response_timeout'] = $this->settings['fetcher_timeout'];
    }
    $client = new $client_class($client_options);
    $connection = new Connection($client);

    // @todo while the retry-on-timeout was very necessary in 2012, we shouldn't
    // be doing retry-loops like this inside an HTTP request. We have
    // 'drunkins_process_more' since 2015 so we should set that instead. On top
    // of that, getData() throws exceptions instead of returning FALSE now. So
    // we'll just comment things out for now. If we start seeing timeouts again
    // that are so many they're unmanageable / just failing the job won't fly...
    // then we'll need to test handling them at that moment.
//    $items = FALSE;
//    $tries_left = !empty($this->settings['fetcher_tries']) ? $this->settings['fetcher_tries'] : 1;
//    while ($items === FALSE && --$tries_left >= 0) {

    $arguments = empty($this->settings['afas_include_empty_fields']) ? array() :
      array('options' => array('Outputoptions' => Connection::GET_OUTPUTOPTIONS_XML_INCLUDE_EMPTY));
    if (!empty($context['afas_return_xml'])) {
      $arguments['options']['Outputmode'] = Connection::GET_OUTPUTMODE_SIMPLEXML;
    }
    if (!empty($this->settings['fetcher_limit'])) {
      $arguments['take'] = $this->settings['fetcher_limit'];
    }
    if (!empty($this->settings['afas_skip'])) {
      $arguments['skip'] = $this->settings['afas_skip'];
    }
    $items = $connection->getData($this->settings['afas_connector'], $this->inputFilter($context), Connection::DATA_TYPE_GET, $arguments);

//      if ($items === FALSE) {
//        // Retry if timeout
//        $error = $connection->getLastCallInfo('error');
//        if (strpos($error, 'cURL ERROR: 28') !== FALSE) {
//          if ($tries_left) {
//            // Do 'error' only because it positions this message in between
//            // the other messages, if this is called from the batch form.
//            $this->log('AFAS timeout occurred; retrying, #@try', array('@try' => $this->settings['fetcher_tries'] - $tries_left), WATCHDOG_ERROR);
//          }
//        }
//        else {
//          $tries_left = 0;
//        }
//      }
//    }

    return $items;
  }

  /**
   * Returns values to filter items on, according to user input. You might
   * call this from your init(), before fetching items in whatever way.
   * (Or check the properties by yourself.)
   *
   * @param array $context
   *   The process context array.
   *
   * @return array
   *   'filters array' that can be used for calling
   *   AfasSoapConnection::getData().
   *
   * @throws \RuntimeException
   *   If the filter or related settings are invalid.
   */
  public function inputFilter(array $context) {
    // This supports 'simple' (array key being the field name) and 'deeper'
    // array arguments.
    $filters = isset($this->settings['afas_filter']) && is_array($this->settings['afas_filter'])
      ? $this->settings['afas_filter'] : array();

    // Add this filter 1 layer deeper than the 'simple' filter arguments.
    // AfasSoapConnection::getData() will sort it out.
    if (!empty($this->extra_filter)) {
      $filters[] = $this->extra_filter;
    }

    // This filter too.
    if ($timestamp = $this->getStartTimestamp()) {
      if (empty($this->settings['afas_updated_field'])) {
        throw new RuntimeException(t('Updated-timestamp field is not specified for AFAS connector, even though we are supposed to be fitering.'));
      }
      else {
        // Since we won't use the timezone in our date expression below, we
        // 'lose' information and have to be careful we get dates in our own
        // timezone instead of UTC.
        $filters[] = array(
          $this->settings['afas_updated_field'] => date('Y-m-d\TH:i:s', $timestamp),
          '#op' => Connection::OP_LARGER_OR_EQUAL,
        );
      }
    }

    foreach ($this->evalFilterInput() as $filter) {
      $filters[] = $filter;
    }

    return $filters;
  }

  /// Private helper functions

  /**
   * Checks for/evaluates a comma separated list of arrays in
   * $this->settings['afas_adv_filter']. This is used for the 'Filter' textarea
   * value in the batch form.
   *
   * @return array
   *   The evaluated filters, or empty array if nothing set / error encountered.
   *
   * @throws \RuntimeException
   *   If this filter does not evaluate to valid PHP array structures.
   */
  private function evalFilterInput() {
    static $filters;

    if (!isset($filters)) {
      $filters = array();

      if (!empty($this->settings['afas_adv_filter'])) {
        $eval = '';
        eval('$eval = array(' . $this->settings['afas_adv_filter'] . ');');
        if (is_array($eval)) {
          foreach ($eval as $filter) {
            if (is_array($filter)) {
              if (!empty($filter)) {
                $filters[] = $filter;
              }
              // Just disregard filter-parts which are empty arrays.
            }
            else {
              // t() in Exception?
              throw new RuntimeException(t("Invalid 'filter' (part) setting passed to inputFilter(): %p", array('%p' => var_export($filter, TRUE))));
            }
          }
        }
        else {
          throw new RuntimeException(t("Invalid 'afas_adv_filter' setting passed to inputFilter(): %p", array('%p' => var_export($this->settings['afas_adv_filter'], TRUE))));
        }
      }
    }
    return $filters;
  }

}
